"""
A polynomial homotopy is a family of polynomial systems with one parameter.
In an artificial parameter homotopy, there is a start and a target system.
There is only one system in a natural parameter homotopy,
where one variable plays the role of the parameter in the homotopy.
The module homotopies exports several functions to set start and
target functions in a homotopy.  There are 36 copy functions:

    copy_{double, double_double, quad_double}
        _{laurent_system, system, solutions}
        _{from, into}_{start, target}

which take no input arguments (other than the verbose level),
and have no return arguments (other than the return value of the call).
Those copy functions are auxiliary to the set functions to define
the homotopies.
"""
from ctypes import c_int32, c_double, pointer
from random import uniform
from cmath import exp, pi
from phcpy.version import get_phcfun
from phcpy.polynomials import number_of_symbols
from phcpy.polynomials import set_double_system
from phcpy.polynomials import get_double_system
from phcpy.polynomials import set_double_laurent_system
from phcpy.polynomials import get_double_laurent_system
from phcpy.polynomials import clear_double_system
from phcpy.polynomials import clear_double_laurent_system
from phcpy.polynomials import set_double_double_system
from phcpy.polynomials import get_double_double_system
from phcpy.polynomials import set_double_double_laurent_system
from phcpy.polynomials import get_double_double_laurent_system
from phcpy.polynomials import clear_double_double_system
from phcpy.polynomials import clear_double_double_laurent_system
from phcpy.polynomials import set_quad_double_system
from phcpy.polynomials import get_quad_double_system
from phcpy.polynomials import set_quad_double_laurent_system
from phcpy.polynomials import get_quad_double_laurent_system
from phcpy.polynomials import clear_quad_double_system
from phcpy.polynomials import clear_quad_double_laurent_system
from phcpy.solutions import set_double_solutions, get_double_solutions
from phcpy.solutions import set_double_double_solutions
from phcpy.solutions import get_double_double_solutions
from phcpy.solutions import set_quad_double_solutions
from phcpy.solutions import get_quad_double_solutions
from phcpy.solutions import clear_double_solutions
from phcpy.solutions import clear_double_double_solutions
from phcpy.solutions import clear_quad_double_solutions
from phcpy.solutions import make_solution, verify

def copy_double_system_from_target(vrblvl=0):
    """
    Copies the target system set in an artificial-parameter homotopy
    in double precision into the system in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_system_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_system_from_target calls phc', end='')
    retval = phc(1, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_system_into_target(vrblvl=0):
    """
    Copies the system set in double precision to the target
    in an artificial-parameter homotopy in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_system_into_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_system_into_target calls phc', end='')
    retval = phc(2, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_laurent_system_from_target(vrblvl=0):
    """
    Copies the target Laurent system set in an artificial-parameter homotopy
    in double precision into the Laurent system in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_laurent_system_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_laurent_system_from_target calls phc', end='')
    retval = phc(786, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_laurent_system_into_target(vrblvl=0):
    """
    Copies the Laurent system set in double precision to the target
    in an artificial-parameter homotopy in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_laurent_system_into_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_laurent_system_into_target calls phc', end='')
    retval = phc(780, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_system_from_target(vrblvl=0):
    """
    Copies the target system set in an artificial-parameter homotopy
    in double double precision into the system in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_system_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_system_from_target calls phc', end='')
    retval = phc(251, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_system_into_target(vrblvl=0):
    """
    Copies the system set in double double precision to the target
    in an artificial-parameter homotopy in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_system_into_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_system_into_target calls phc', end='')
    retval = phc(252, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_laurent_system_from_target(vrblvl=0):
    """
    Copies the target Laurent system set in an artificial-parameter homotopy
    in double double precision into the Laurent system
    in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_laurent_system_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_laurent_system_from_target calls phc', \
            end='')
    retval = phc(787, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_laurent_system_into_target(vrblvl=0):
    """
    Copies the system set in double double precision to the target
    in an artificial-parameter homotopy in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_laurent_system_into_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_laurent_system_into_target calls phc', \
            end='')
    retval = phc(781, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_system_from_target(vrblvl=0):
    """
    Copies the target system set in an artificial-parameter homotopy
    in quad double precision into the system in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_system_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_system_from_target calls phc', end='')
    retval = phc(261, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_system_into_target(vrblvl=0):
    """
    Copies the system set in quad double precision to the target
    system in an artificial-parameter homotopy in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_system_into_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_system_into_target calls phc', end='')
    retval = phc(262, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_laurent_system_from_target(vrblvl=0):
    """
    Copies the target Laurent system set in an artificial-parameter homotopy
    in quad double precision into the Laurent system in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_laurent_system_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_laurent_system_from_target calls phc', \
            end='')
    retval = phc(788, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_laurent_system_into_target(vrblvl=0):
    """
    Copies the Laurent system set in quad double precision to the target
    system in an artificial-parameter homotopy in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_laurent_system_into_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_laurent_system_into_target calls phc', \
            end='')
    retval = phc(782, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_system_from_start(vrblvl=0):
    """
    Copies the start system set in an artificial-parameter homotopy
    in double precision into the system in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_system_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_system_from_start calls phc', end='')
    retval = phc(3, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_system_into_start(vrblvl=0):
    """
    Copies the system set in double precision to the start
    system in an artificial-parameter homotopy in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_system_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_system_into_start calls phc', end='')
    retval = phc(4, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_laurent_system_from_start(vrblvl=0):
    """
    Copies the start Laurent system set in an artificial-parameter homotopy
    in double precision into the Laurent system in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_laurent_system_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_laurent_system_from_start calls phc', end='')
    retval = phc(783, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_laurent_system_into_start(vrblvl=0):
    """
    Copies the Laurent system set in double precision to the start
    system in an artificial-parameter homotopy in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_laurent_system_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_laurent_system_into_start calls phc', end='')
    retval = phc(777, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_system_from_start(vrblvl=0):
    """
    Copies the start system set in an artificial-parameter homotopy
    in double double precision into the system in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_system_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_double_system_from_start calls phc', end='')
    retval = phc(253, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_system_into_start(vrblvl=0):
    """
    Copies the system set in double double precision to the start
    in an artificial-parameter homotopy in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_system_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_double_system_into_start calls phc', end='')
    retval = phc(254, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_laurent_system_from_start(vrblvl=0):
    """
    Copies the start Laurent system set in an artificial-parameter homotopy
    in double double precision into the Laurent system
    in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_laurent_system_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_double_laurent_system_from_start calls phc', \
            end='')
    retval = phc(784, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_laurent_system_into_start(vrblvl=0):
    """
    Copies the Laurent system set in double double precision to the start
    in an artificial-parameter homotopy in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_laurent_system_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_double_laurent_system_into_start calls phc', \
            end='')
    retval = phc(778, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_system_from_start(vrblvl=0):
    """
    Copies the start system set in an artificial-parameter homotopy
    in quad double precision into the system in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_system_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_system_from_start calls phc', end='')
    retval = phc(263, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_system_into_start(vrblvl=0):
    """
    Copies the system set in quad double precision to the start
    in an artificial-parameter homotopy in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_system_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_system_into_start calls phc', end='')
    retval = phc(264, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_laurent_system_from_start(vrblvl=0):
    """
    Copies the start Laurent system set in an artificial-parameter homotopy
    in quad double precision into the Laurent system
    in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_laurent_system_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_laurent_system_from_start calls phc', \
            end='')
    retval = phc(785, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_laurent_system_into_start(vrblvl=0):
    """
    Copies the Laurent system set in quad double precision to the start
    in an artificial-parameter homotopy in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_laurent_system_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_laurent_system_into_start calls phc', \
            end='')
    retval = phc(779, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_solutions_from_start(vrblvl=0):
    """
    Copies the start solutions in an artificial-parameter homotopy
    in double precision to the solutions in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_solutions_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_solutions_from_start calls phc', end='')
    retval = phc(7, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_solutions_into_start(vrblvl=0):
    """
    Copies the solutions set in double precision to the start
    solutions in an artificial-parameter homotopy in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_solutions_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_solutions_into_start calls phc', end='')
    retval = phc(8, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_solutions_from_start(vrblvl=0):
    """
    Copies the start solutions in an artificial-parameter homotopy
    in double double precision to the solutions in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_solutions_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_double_solutions_from_start calls phc', end='')
    retval = phc(257, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_solutions_into_start(vrblvl=0):
    """
    Copies the solutions set in double double precision to the start
    solutions in an artificial-parameter homotopy in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_solutions_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_double_solutions_into_start calls phc', end='')
    retval = phc(258, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_solutions_from_start(vrblvl=0):
    """
    Copies the start solutions in an artificial-parameter homotopy
    in quad double precision to the solutions in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_solutions_from_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_solutions_from_start calls phc', end='')
    retval = phc(267, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_solutions_into_start(vrblvl=0):
    """
    Copies the solutions set in quad double precision to the start
    solutions in an artificial-parameter homotopy in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_solutions_into_start ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_solutions_into_start calls phc', end='')
    retval = phc(268, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_solutions_from_target(vrblvl=0):
    """
    Copies the target solutions in an artificial-parameter homotopy
    in double precision to the solutions in double precision.
    """
    if vrblvl > 0:
        print('in copy_double_solutions_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_solutions_from_target calls phc', end='')
    retval = phc(5, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_solutions_into_target(vrblvl=0):
    """
    Copies the solutions set in double precision to the target
    solutions in an artificial-parameter homotopy in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_solutions_into_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_solutions_into_target calls phc', end='')
    retval = phc(6, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_solutions_from_target(vrblvl=0):
    """
    Copies the target solutions in an artificial-parameter homotopy
    in double precision to the solutions in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_solutions_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_double_solutions_from_target calls phc', end='')
    retval = phc(255, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_double_double_solutions_into_target(vrblvl=0):
    """
    Copies the solutions set in double double precision to the target
    solutions in an artificial-parameter homotopy in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_double_double_target_solutions ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_double_double_target_solutions calls phc', end='')
    retval = phc(256, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_solutions_from_target(vrblvl=0):
    """
    Copies the target solutions in an artificial-parameter homotopy
    in quad double precision to the solutions in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_solutions_from_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_solutions_from_target calls phc', end='')
    retval = phc(265, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_quad_double_solutions_into_target(vrblvl=0):
    """
    Copies the solutions set in quad double precision to the target
    solutions in an artificial-parameter homotopy in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in copy_quad_double_solutions_into_target ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_quad_double_solutions_into_target calls phc', end='')
    retval = phc(266, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_target_system(pols, vrblvl=0):
    """
    Sets the target system in an artificial parameter homotopy
    in double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_target_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_double_system(nvr, pols, vrblvl)
    return copy_double_system_into_target(vrblvl)

def set_double_double_target_system(pols, vrblvl=0):
    """
    Sets the target system in an artificial parameter homotopy
    in double double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_double_target_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_double_double_system(nvr, pols, vrblvl)
    return copy_double_double_system_into_target(vrblvl)

def set_quad_double_target_system(pols, vrblvl=0):
    """
    Sets the target system in an artificial parameter homotopy
    in quad double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_quad_double_target_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_quad_double_system(nvr, pols, vrblvl)
    return copy_quad_double_system_into_target(vrblvl)

def set_double_laurent_target_system(pols, vrblvl=0):
    """
    Sets the target Laurent system in an artificial parameter homotopy
    in double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_laurent_target_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_double_laurent_system(nvr, pols, vrblvl)
    return copy_double_laurent_system_into_target(vrblvl)

def set_double_double_laurent_target_system(pols, vrblvl=0):
    """
    Sets the target Laurent system in an artificial parameter homotopy
    in double double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_double_laurent_target_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_double_double_laurent_system(nvr, pols, vrblvl)
    return copy_double_double_laurent_system_into_target(vrblvl)

def set_quad_double_laurent_target_system(pols, vrblvl=0):
    """
    Sets the target Laurent system in an artificial parameter homotopy
    in quad double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_quad_double_target_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_quad_double_laurent_system(nvr, pols, vrblvl)
    return copy_quad_double_laurent_system_into_target(vrblvl)

def set_double_start_system(pols, vrblvl=0):
    """
    Sets the start system in an artificial parameter homotopy
    in double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_start_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_double_system(nvr, pols, vrblvl)
    return copy_double_system_into_start(vrblvl)

def set_double_double_start_system(pols, vrblvl=0):
    """
    Sets the start system in an artificial parameter homotopy
    in double double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_double_start_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_double_double_system(nvr, pols, vrblvl)
    return copy_double_double_system_into_start(vrblvl)

def set_quad_double_start_system(pols, vrblvl=0):
    """
    Sets the start system in an artificial parameter homotopy
    in quad double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_quad_double_start_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_quad_double_system(nvr, pols, vrblvl)
    return copy_quad_double_system_into_start(vrblvl)

def set_double_laurent_start_system(pols, vrblvl=0):
    """
    Sets the start Laurent system in an artificial parameter homotopy
    in double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_start_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_double_laurent_system(nvr, pols, vrblvl)
    return copy_double_laurent_system_into_start(vrblvl)

def set_double_double_laurent_start_system(pols, vrblvl=0):
    """
    Sets the start Laurent system in an artificial parameter homotopy
    in double double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_double_laurent_start_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_double_double_laurent_system(nvr, pols, vrblvl)
    return copy_double_double_laurent_system_into_start(vrblvl)

def set_quad_double_laurent_start_system(pols, vrblvl=0):
    """
    Sets the start Laurent system in an artificial parameter homotopy
    in quad double precision to the list of polynomials in pols,
    which is assumed to be square.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_quad_double_laurent_start_system, with pols :')
        for pol in pols:
            print(pol)
    nvr = number_of_symbols(pols, vrblvl)
    set_quad_double_laurent_system(nvr, pols, vrblvl)
    return copy_quad_double_laurent_system_into_start(vrblvl)

def set_double_start_solutions(nvr, sols, vrblvl=0):
    """
    Sets the start solutions in an artificial parameter homotopy
    in double precision to the list of solutions in sols,
    where the number of variables in nvr must match the
    dimension of the start system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_start_solutions, with nvr :', nvr)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    clear_double_solutions(vrblvl)
    set_double_solutions(nvr, sols, vrblvl)
    return copy_double_solutions_into_start(vrblvl)

def set_double_double_start_solutions(nvr, sols, vrblvl=0):
    """
    Sets the start solutions in an artificial parameter homotopy
    in double double precision to the list of solutions in sols,
    where the number of variables in nvr must match the
    dimension of the start system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_double_start_solutions, with nvr :', nvr)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    clear_double_double_solutions(vrblvl)
    set_double_double_solutions(nvr, sols, vrblvl)
    return copy_double_double_solutions_into_start(vrblvl)

def set_quad_double_start_solutions(nvr, sols, vrblvl=0):
    """
    Sets the start solutions in an artificial parameter homotopy
    in quad double precision to the list of solutions in sols,
    where the number of variables in nvr must match the
    dimension of the start system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_quad_double_start_solutions, with nvr :', nvr)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    clear_quad_double_solutions(vrblvl)
    set_quad_double_solutions(nvr, sols, vrblvl)
    return copy_quad_double_solutions_into_start(vrblvl)

def get_double_target_solutions(vrblvl=0):
    """
    Returns the list of target solutions computed in double precision.
    """
    if vrblvl > 0:
        print('in get_double_target_solutions ...')
    clear_double_solutions(vrblvl)
    copy_double_solutions_from_target(vrblvl)
    sols = get_double_solutions(vrblvl)
    if vrblvl > 0:
        print('the target solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    return sols

def get_double_double_target_solutions(vrblvl=0):
    """
    Returns the list of target solutions computed
    in double double precision.
    """
    if vrblvl > 0:
        print('in get_double_double_target_solutions ...')
    clear_double_double_solutions(vrblvl)
    copy_double_double_solutions_from_target(vrblvl)
    sols = get_double_double_solutions(vrblvl)
    if vrblvl > 0:
        print('the target solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    return sols

def get_quad_double_target_solutions(vrblvl=0):
    """
    Returns the list of target solutions computed
    in quad double precision.
    """
    if vrblvl > 0:
        print('in get_quad_double_target_solutions ...')
    clear_quad_double_solutions(vrblvl)
    copy_quad_double_solutions_from_target(vrblvl)
    sols = get_quad_double_solutions(vrblvl)
    if vrblvl > 0:
        print('the target solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    return sols

def set_double_target_solutions(nvr, sols, vrblvl=0):
    """
    Sets the target solutions in an artificial parameter homotopy
    in double precision to the list of solutions in sols,
    where the number of variables in nvr must match the
    dimension of the start system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_target_solutions, with nvr :', nvr)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    clear_double_solutions(vrblvl)
    set_double_solutions(nvr, sols, vrblvl)
    return copy_double_solutions_into_target(vrblvl)

def set_double_double_target_solutions(nvr, sols, vrblvl=0):
    """
    Sets the target solutions in an artificial parameter homotopy
    in double double precision to the list of solutions in sols,
    where the number of variables in nvr must match the
    dimension of the start system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_double_double_target_solutions, with nvr :', nvr)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    clear_double_double_solutions(vrblvl)
    set_double_double_solutions(nvr, sols, vrblvl)
    return copy_double_double_solutions_into_target(vrblvl)

def set_quad_double_target_solutions(nvr, sols, vrblvl=0):
    """
    Sets the target solutions in an artificial parameter homotopy
    in quad double precision to the list of solutions in sols,
    where the number of variables in nvr must match the
    dimension of the start system.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_quad_double_target_solutions, with nvr :', nvr)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    clear_quad_double_solutions(vrblvl)
    set_quad_double_solutions(nvr, sols, vrblvl)
    return copy_quad_double_solutions_into_target(vrblvl)

def set_double_homotopy(gamma=0, pwt=2, vrblvl=0):
    """
    After the target and start system are set in double precision,
    the homotopy is constructed with either a random gamma constant,
    or with the given complex value of gamma.
    The power of the continuation parameter is given by pwt.
    The gamma used to make the homotopy is returned.
    """
    if vrblvl > 0:
        print('in set_double_homotopy, with power :', pwt)
        print('gamma :', gamma)
    phc = get_phcfun(vrblvl-1)
    apwt = pointer(c_int32(pwt))
    bbb = pointer(c_int32(0))
    if gamma != 0:
        usegamma = gamma
    else:
        angle = uniform(0, 2*pi)
        usegamma = exp(angle*complex(0, 1))
        if vrblvl > 0:
            print('random gamma :', usegamma)
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(usegamma.real)
    c_gamma[1] = c_double(usegamma.imag)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_homotopy calls phc', end='')
    retval = phc(153, apwt, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    if vrblvl > 0:
        print('-> set_double_homotopy calls phc', end='')
        print(' to set gamma', end='')
    aprc = pointer(c_int32(1))
    retval = phc(996, aprc, bbb, ptr_gamma, vrb) # set gamma
    if vrblvl > 0:
        print(', return value :', retval)
    return usegamma

def set_double_double_homotopy(gamma=0, pwt=2, vrblvl=0):
    """
    After the target and start system are set in double double precision,
    the homotopy is constructed with either a random gamma constant,
    or with the given complex value of gamma.
    The power of the continuation parameter is given by pwt.
    The gamma used to make the homotopy is returned.
    """
    if vrblvl > 0:
        print('in set_double_double_homotopy, with power :', pwt)
        print('gamma :', gamma)
    phc = get_phcfun(vrblvl-1)
    apwt = pointer(c_int32(pwt))
    bbb = pointer(c_int32(0))
    if gamma != 0:
        usegamma = gamma
    else:
        angle = uniform(0, 2*pi)
        usegamma = exp(angle*complex(0, 1))
        if vrblvl > 0:
            print('random gamma :', usegamma)
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(usegamma.real)
    c_gamma[1] = c_double(usegamma.imag)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_homotopy calls phc', end='')
    retval = phc(173, apwt, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    if vrblvl > 0:
        print('-> set_double_double_homotopy calls phc', end='')
        print(' to set gamma', end='')
    aprc = pointer(c_int32(2))
    retval = phc(996, aprc, bbb, ptr_gamma, vrb) # set gamma
    if vrblvl > 0:
        print(', return value :', retval)
    return usegamma

def set_quad_double_homotopy(gamma=0, pwt=2, vrblvl=0):
    """
    After the target and start system are set in quad double precision,
    the homotopy is constructed with either a random gamma constant,
    or with the given complex value of gamma.
    The power of the continuation parameter is given by pwt.
    The gamma used to make the homotopy is returned.
    """
    if vrblvl > 0:
        print('in set_quad_double_homotopy, with power :', pwt)
        print('gamma :', gamma)
    phc = get_phcfun(vrblvl-1)
    apwt = pointer(c_int32(pwt))
    bbb = pointer(c_int32(0))
    if gamma != 0:
        usegamma = gamma
    else:
        angle = uniform(0, 2*pi)
        usegamma = exp(angle*complex(0, 1))
        if vrblvl > 0:
            print('random gamma :', usegamma)
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(usegamma.real)
    c_gamma[1] = c_double(usegamma.imag)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> set_quad_double_homotopy calls phc', end='')
    retval = phc(183, apwt, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    if vrblvl > 0:
        print('-> set_quad_double_homotopy calls phc', end='')
        print(' to set gamma', end='')
    aprc = pointer(c_int32(3))
    retval = phc(996, aprc, bbb, ptr_gamma, vrb) # set gamma
    if vrblvl > 0:
        print(', return value :', retval)
    return usegamma

def set_double_laurent_homotopy(gamma=0, pwt=2, vrblvl=0):
    """
    After the target and start Laurent system are set in double precision,
    the homotopy is constructed with either a random gamma constant,
    or with the given complex value of gamma.
    The power of the continuation parameter is given by pwt.
    The gamma used to make the Laurent homotopy is returned.
    """
    if vrblvl > 0:
        print('in set_double_laurent_homotopy, with power :', pwt)
        print('gamma :', gamma)
    phc = get_phcfun(vrblvl-1)
    apwt = pointer(c_int32(pwt))
    bbb = pointer(c_int32(0))
    if gamma != 0:
        usegamma = gamma
    else:
        angle = uniform(0, 2*pi)
        usegamma = exp(angle*complex(0, 1))
        if vrblvl > 0:
            print('random gamma :', usegamma)
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(usegamma.real)
    c_gamma[1] = c_double(usegamma.imag)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_laurent_homotopy calls phc', end='')
    retval = phc(921, apwt, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    if vrblvl > 0:
        print('-> set_double_laurent_homotopy calls phc', end='')
        print(' to set gamma', end='')
    aprc = pointer(c_int32(1))
    retval = phc(996, aprc, bbb, ptr_gamma, vrb) # set gamma
    if vrblvl > 0:
        print(', return value :', retval)
    return usegamma

def set_double_double_laurent_homotopy(gamma=0, pwt=2, vrblvl=0):
    """
    After the target and start Laurent system are set 
    in double double precision,
    the homotopy is constructed with either a random gamma constant,
    or with the given complex value of gamma.
    The power of the continuation parameter is given by pwt.
    The gamma used to make the Laurent homotopy is returned.
    """
    if vrblvl > 0:
        print('in set_double_double_laurent_homotopy, with power :', pwt)
        print('gamma :', gamma)
    phc = get_phcfun(vrblvl-1)
    apwt = pointer(c_int32(pwt))
    bbb = pointer(c_int32(0))
    if gamma != 0:
        usegamma = gamma
    else:
        angle = uniform(0, 2*pi)
        usegamma = exp(angle*complex(0, 1))
        if vrblvl > 0:
            print('random gamma :', usegamma)
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(usegamma.real)
    c_gamma[1] = c_double(usegamma.imag)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_double_double_laurent_homotopy calls phc', end='')
    retval = phc(922, apwt, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    if vrblvl > 0:
        print('-> set_double_double_laurent_homotopy calls phc', end='')
        print(' to set gamma', end='')
    aprc = pointer(c_int32(2))
    retval = phc(996, aprc, bbb, ptr_gamma, vrb) # set gamma
    if vrblvl > 0:
        print(', return value :', retval)
    return usegamma

def set_quad_double_laurent_homotopy(gamma=0, pwt=2, vrblvl=0):
    """
    After the target and start Laurent system are set 
    in quad double precision,
    the homotopy is constructed with either a random gamma constant,
    or with the given complex value of gamma.
    The power of the continuation parameter is given by pwt.
    The gamma used to make the Laurent homotopy is returned.
    """
    if vrblvl > 0:
        print('in set_quad_double_laurent_homotopy, with power :', pwt)
        print('gamma :', gamma)
    phc = get_phcfun(vrblvl-1)
    apwt = pointer(c_int32(pwt))
    bbb = pointer(c_int32(0))
    if gamma != 0:
        usegamma = gamma
    else:
        angle = uniform(0, 2*pi)
        usegamma = exp(angle*complex(0, 1))
        if vrblvl > 0:
            print('random gamma :', usegamma)
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(usegamma.real)
    c_gamma[1] = c_double(usegamma.imag)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_quad_double_laurent_homotopy calls phc', end='')
    retval = phc(923, apwt, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    if vrblvl > 0:
        print('-> set_quad_double_laurent_homotopy calls phc', end='')
        print(' to set gamma', end='')
    aprc = pointer(c_int32(3))
    retval = phc(996, aprc, bbb, ptr_gamma, vrb) # set gamma
    if vrblvl > 0:
        print(', return value :', retval)
    return usegamma

def clear_double_homotopy(vrblvl=0):
    """
    Clears the homotopy set in double precision.
    """
    if vrblvl > 0:
        print('in clear_double_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> clear_double_homotopy calls phc', end='')
    retval = phc(154, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_double_homotopy(vrblvl=0):
    """
    Clears the homotopy set in double double precision.
    """
    if vrblvl > 0:
        print('in clear_double_double_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> clear_double_double_homotopy calls phc', end='')
    retval = phc(174, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_quad_double_homotopy(vrblvl=0):
    """
    Clears the homotopy set in quad double precision.
    """
    if vrblvl > 0:
        print('in clear_quad_double_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> clear_quad_double_homotopy calls phc', end='')
    retval = phc(184, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_laurent_homotopy(vrblvl=0):
    """
    Clears the homotopy set in double precision.
    """
    if vrblvl > 0:
        print('in clear_double_laurent_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_laurent_homotopy calls phc', end='')
    retval = phc(924, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_double_laurent_homotopy(vrblvl=0):
    """
    Clears the Laurent homotopy set in double double precision.
    """
    if vrblvl > 0:
        print('in clear_double_double_laurent_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_double_laurent_homotopy calls phc', end='')
    retval = phc(925, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_quad_double_laurent_homotopy(vrblvl=0):
    """
    Clears the Laurent homotopy set in quad double precision.
    """
    if vrblvl > 0:
        print('in clear_quad_double_laurent_homotopy ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_quad_double_laurent_homotopy calls phc', end='')
    retval = phc(926, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def example_start_system(vrblvl=0):
    """
    Returns an example of a start system.
    """
    if vrblvl > 0:
        print('in example_start_system ...')
    pols = ['x^2 - 1;', 'y - 1;']
    names = ['x', 'y']
    sol1 = make_solution(names, [1, 1], vrblvl=vrblvl-1)
    sol2 = make_solution(names, [-1, 1], vrblvl=vrblvl-1)
    sols = [sol1, sol2]
    return (pols, sols)

def example_target_system(vrblvl=0):
    """
    Returns an example of a target system.
    """
    if vrblvl > 0:
        print('in example_start_system ...')
    pols = ['(x - 2)*(y - 2);', 'x + y - 2;']
    names = ['x', 'y']
    sol1 = make_solution(names, [2, 0], vrblvl=vrblvl-1)
    sol2 = make_solution(names, [0, 2], vrblvl=vrblvl-1)
    sols = [sol1, sol2]
    return (pols, sols)

def test_double_start_system(vrblvl=0):
    """
    Tests the definition of a start system in double precision.
    """
    if vrblvl > 0:
        print('in test_double_start_system ...')
    (pols, sols) = example_start_system(vrblvl)
    if vrblvl > 0:
        print('the start system :')
        for pol in pols:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(pols, sols, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the error of the start solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_double_start_system(pols, vrblvl-1)
    clear_double_system(vrblvl-1)
    copy_double_system_from_start(vrblvl-1)
    pols = get_double_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_double_start_solutions(2, sols, vrblvl-1)
    clear_double_solutions(vrblvl-1)
    copy_double_solutions_from_start(vrblvl-1)
    sols = get_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_double_double_start_system(vrblvl=0):
    """
    Tests the definition of a start system in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_start_system ...')
    (pols, sols) = example_start_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the start solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_double_double_start_system(pols, vrblvl)
    clear_double_double_system(vrblvl-1)
    copy_double_double_system_from_start(vrblvl)
    pols = get_double_double_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_double_double_start_solutions(2, sols, vrblvl)
    clear_double_double_solutions(vrblvl-1)
    copy_double_double_solutions_from_start(vrblvl)
    sols = get_double_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_quad_double_start_system(vrblvl=0):
    """
    Tests the definition of a start system in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_start_system ...')
    (pols, sols) = example_start_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the start solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_quad_double_start_system(pols, vrblvl)
    clear_quad_double_system(vrblvl-1)
    copy_quad_double_system_from_start(vrblvl)
    pols = get_quad_double_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_quad_double_start_solutions(2, sols, vrblvl)
    clear_quad_double_solutions(vrblvl-1)
    copy_quad_double_solutions_from_start(vrblvl)
    sols = get_quad_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_double_target_system(vrblvl=0):
    """
    Tests the definition of a target system in double precision.
    """
    if vrblvl > 0:
        print('in test_double_target_system ...')
    (pols, sols) = example_target_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the target solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_double_target_system(pols, vrblvl)
    clear_double_system(vrblvl-1)
    copy_double_system_from_target(vrblvl)
    pols = get_double_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_double_target_solutions(2, sols, vrblvl)
    clear_double_solutions(vrblvl)
    sols = get_double_target_solutions(vrblvl)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_double_double_target_system(vrblvl=0):
    """
    Tests the definition of a target system in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_target_system ...')
    (pols, sols) = example_target_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the target solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_double_double_target_system(pols, vrblvl)
    clear_double_double_system(vrblvl-1)
    copy_double_double_system_from_target(vrblvl)
    pols = get_double_double_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_double_double_target_solutions(2, sols, vrblvl)
    clear_double_double_solutions(vrblvl)
    sols = get_double_double_target_solutions(vrblvl)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_quad_double_target_system(vrblvl=0):
    """
    Tests the definition of a target system in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_target_system ...')
    (pols, sols) = example_target_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the target solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_quad_double_target_system(pols, vrblvl)
    clear_quad_double_system(vrblvl-1)
    copy_quad_double_system_from_target(vrblvl)
    pols = get_quad_double_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_quad_double_target_solutions(2, sols, vrblvl)
    clear_quad_double_solutions(vrblvl)
    sols = get_quad_double_target_solutions(vrblvl)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_double_laurent_start_system(vrblvl=0):
    """
    Tests the definition of a start system in double precision,
    set as a Laurent system.
    """
    if vrblvl > 0:
        print('in test_double_laurent_start_system ...')
    (pols, sols) = example_start_system(vrblvl)
    if vrblvl > 0:
        print('the start system :')
        for pol in pols:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(pols, sols, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the error of the start solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_double_laurent_start_system(pols, vrblvl-1)
    clear_double_laurent_system(vrblvl-1)
    copy_double_laurent_system_from_start(vrblvl-1)
    pols = get_double_laurent_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_double_start_solutions(2, sols, vrblvl-1)
    clear_double_solutions(vrblvl-1)
    copy_double_solutions_from_start(vrblvl-1)
    sols = get_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_double_double_laurent_start_system(vrblvl=0):
    """
    Tests the definition of a start system in double double precision,
    set as Laurent system.
    """
    if vrblvl > 0:
        print('in test_double_double_laurent_start_system ...')
    (pols, sols) = example_start_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the start solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_double_double_laurent_start_system(pols, vrblvl)
    clear_double_double_laurent_system(vrblvl-1)
    copy_double_double_laurent_system_from_start(vrblvl)
    pols = get_double_double_laurent_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_double_double_start_solutions(2, sols, vrblvl)
    clear_double_double_solutions(vrblvl-1)
    copy_double_double_solutions_from_start(vrblvl)
    sols = get_double_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_quad_double_laurent_start_system(vrblvl=0):
    """
    Tests the definition of a start system in quad double precision,
    set as Laurent system.
    """
    if vrblvl > 0:
        print('in test_quad_double_laurent_start_system ...')
    (pols, sols) = example_start_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the start solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_quad_double_laurent_start_system(pols, vrblvl)
    clear_quad_double_laurent_system(vrblvl-1)
    copy_quad_double_laurent_system_from_start(vrblvl)
    pols = get_quad_double_laurent_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_quad_double_start_solutions(2, sols, vrblvl)
    clear_quad_double_solutions(vrblvl-1)
    copy_quad_double_solutions_from_start(vrblvl)
    sols = get_quad_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_double_laurent_target_system(vrblvl=0):
    """
    Tests the definition of a target system in double precision,
    set as Laurent system.
    """
    if vrblvl > 0:
        print('in test_double_laurent_target_system ...')
    (pols, sols) = example_target_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the target solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_double_laurent_target_system(pols, vrblvl)
    clear_double_laurent_system(vrblvl-1)
    copy_double_laurent_system_from_target(vrblvl)
    pols = get_double_laurent_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_double_target_solutions(2, sols, vrblvl)
    clear_double_solutions(vrblvl)
    sols = get_double_target_solutions(vrblvl)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_double_double_laurent_target_system(vrblvl=0):
    """
    Tests the definition of a target system in double double precision,
    set as Laurent system.
    """
    if vrblvl > 0:
        print('in test_double_double_laurent_target_system ...')
    (pols, sols) = example_target_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the target solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_double_double_laurent_target_system(pols, vrblvl)
    clear_double_double_laurent_system(vrblvl-1)
    copy_double_double_laurent_system_from_target(vrblvl)
    pols = get_double_double_laurent_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_double_double_target_solutions(2, sols, vrblvl)
    clear_double_double_solutions(vrblvl)
    sols = get_double_double_target_solutions(vrblvl)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def test_quad_double_laurent_target_system(vrblvl=0):
    """
    Tests the definition of a target system in quad double precision,
    set as Laurent system.
    """
    if vrblvl > 0:
        print('in test_quad_double_laurent_target_system ...')
    (pols, sols) = example_target_system(vrblvl)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error of the target solutions :', err)
    fail = int(abs(err.real) + abs(err.imag) > 1.0e-8)
    set_quad_double_laurent_target_system(pols, vrblvl)
    clear_quad_double_laurent_system(vrblvl-1)
    copy_quad_double_laurent_system_from_target(vrblvl)
    pols = get_quad_double_laurent_system(vrblvl-1)
    if vrblvl > 0:
        print('retrieved polynomials :')
        for pol in pols:
            print(pol)
    fail = fail + int(len(pols) != 2)
    set_quad_double_target_solutions(2, sols, vrblvl)
    clear_quad_double_solutions(vrblvl)
    sols = get_quad_double_target_solutions(vrblvl)
    if vrblvl > 0:
        print('retrieved solutions :')
        for (idx, sol) in enumerate(sols):
            print('solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(sols) != 2)
    return fail

def main():
    """
    Runs some tests.
    """
    lvl = 1
    fail = test_double_start_system(lvl)
    fail = fail + test_double_double_start_system(lvl)
    fail = fail + test_quad_double_start_system(lvl)
    fail = fail + test_double_target_system(lvl)
    fail = fail + test_double_double_target_system(lvl)
    fail = fail + test_quad_double_target_system(lvl)
    fail = fail + test_double_laurent_start_system(lvl)
    fail = fail + test_double_double_laurent_start_system(lvl)
    fail = fail + test_quad_double_laurent_start_system(lvl)
    fail = fail + test_double_laurent_target_system(lvl)
    fail = fail + test_double_double_laurent_target_system(lvl)
    fail = fail + test_quad_double_laurent_target_system(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=='__main__':
    main()



"""
This module provides the data interface to PHCpack.
The user gives as input string representations of polynomials or solutions
to the interface functions which store the data.
"""

def store_standard_tableau(poltab, verbose=False):
    r"""
    Stores the polynomial system given in the list of lists *poltab* in
    the container for systems with coefficients in standard double precision.
    Every polynomial in the system is represented by a list of tuples.
    A monomial is represented by a 2-tuple:

    1. the coefficient of the monomial is a complex number,

    2. the exponent are a tuple of natural numbers.

    For example, the system x^2 - y = 0, x^3 - z = 0 is represented as
    [[((1+0j), (2, 0, 0)), ((-1+0j), (0, 1, 0))], \
    [((1+0j), (3, 0, 0)), ((-1+0j), (0, 0, 1))]].
    """
    from phcpy.phcpy2c3 import py2c_tabform_store_standard_tableau as store
    neq = len(poltab)                               # number of equations
    if verbose:
        print('number of equations :', neq)
    if neq > 0:
        nvr = len(poltab[0][0][1])                  # number of variables
        if verbose:
            print('number of variables :', nvr)
        nbt = [len(pol) for pol in poltab]          # number of terms
        if verbose:
            print('number of terms :', nbt)
        cff = [[c for (c, e) in p] for p in poltab] # coefficients
        if verbose:
            print('the coefficients on input :', cff)
        flatcff = sum(cff, [])                      # flatten list of lists
        frimcff = sum([[x.real, x.imag] for x in flatcff], [])
        if verbose:
            print('flat list of coefficients :', frimcff)
        xps = [[e for (c, e) in p] for p in poltab] # exponents
        if verbose:
            print('the exponents on input :', xps)
        txps = sum(xps, [])                         # list of tuples
        lxps = [list(x) for x in txps]              # list of lists
        flatxps = sum(lxps, [])                     # flatten list of lists
        if verbose:
            print('flat list of exponents :', flatxps)
        strnbt = str(nbt)
        strcff = str(frimcff)
        strxps = str(flatxps)
        fail = store(neq, nvr, len(strnbt), strnbt, \
                     len(strcff), strcff, len(strxps), strxps, int(verbose))

def load_standard_tableau(verbose=False):
    """
    Returns the tableau form of the system stored in the container for
    double precision coefficients.
    """
    from phcpy.phcpy2c3 import py2c_tabform_load_standard_tableau as load
    neq, nvr, nbterms, coefficients, exponents = load(int(verbose))
    nbt = eval(nbterms)
    cff = eval(coefficients)
    xps = eval(exponents)
    xpsidx = 0
    cffidx = 0
    tableau = []
    for i in range(neq):
        pol = []
        for j in range(nbt[i]):
            trm = []
            for k in range(nvr):
                trm.append(xps[xpsidx])
                xpsidx = xpsidx + 1
            cfftrm = complex(cff[cffidx], cff[cffidx+1])
            cffidx = cffidx + 2
            pol.append((cfftrm ,tuple(trm)))
        tableau.append(pol)
    return (neq, nvr, tableau)

def store_standard_system(polsys, **nbvar):
    r"""
    Stores the polynomials represented by the list of strings in *polsys* into
    the container for systems with coefficients in standard double precision.
    The number of variables is an optional argument given in *nbvar*.
    If *nbvar* is omitted, then the system is assumed to be square.
    Otherwise, suppose the number of variables equals 2 and pols is the list
    of polynomials, then the call **store_standard_system(pols, nbvar=2)**
    will store the polynomials in pols in the standard systems container.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_standard_system
    from phcpy.phcpy2c3 \
        import py2c_syscon_initialize_number_of_standard_polynomials
    from phcpy.phcpy2c3 import py2c_syscon_store_standard_polynomial
    py2c_syscon_clear_standard_system()
    dim = len(polsys)
    fail = 0
    py2c_syscon_initialize_number_of_standard_polynomials(dim)
    for cnt in range(0, dim):
        pol = polsys[cnt]
        nchar = len(pol)
        if(len(nbvar) == 0):
            fail = py2c_syscon_store_standard_polynomial(nchar, dim, cnt+1, pol)
        else:
            nvr = list(nbvar.values())[0]
            fail = py2c_syscon_store_standard_polynomial(nchar, nvr, cnt+1, pol)
        if(fail != 0):
            break
    return fail

def store_dobldobl_system(polsys, **nbvar):
    r"""
    Stores the polynomials represented by the list of strings in *polsys*
    into the systems container for double double arithmetic.
    The number of variables is an optional argument given in *nbvar*.
    If *nbvar* is omitted, then the system is assumed to be square.
    Otherwise, suppose the number of variables equals 2 and pols is the list
    of polynomials, then the call **store_dobldobl_system(pols, nbvar=2)**
    will store the polynomials in pols in the dobldobl systems container.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_dobldobl_system
    from phcpy.phcpy2c3 \
        import py2c_syscon_initialize_number_of_dobldobl_polynomials
    from phcpy.phcpy2c3 import py2c_syscon_store_dobldobl_polynomial
    py2c_syscon_clear_dobldobl_system()
    dim = len(polsys)
    fail = 0
    py2c_syscon_initialize_number_of_dobldobl_polynomials(dim)
    for cnt in range(0, dim):
        pol = polsys[cnt]
        nchar = len(pol)
        if(len(nbvar) == 0):
            fail = py2c_syscon_store_dobldobl_polynomial(nchar, dim, cnt+1, pol)
        else:
            nvr = list(nbvar.values())[0]
            fail = py2c_syscon_store_dobldobl_polynomial(nchar, nvr, cnt+1, pol)
        if(fail != 0):
            break
    return fail

def store_quaddobl_system(polsys, **nbvar):
    r"""
    Stores the polynomials represented by the list of strings in *polsys*
    into the systems container for quad double arithmetic.
    The number of variables is an optional argument given in *nbvar*.
    If *nbvar* is omitted, then the system is assumed to be square.
    Otherwise, suppose the number of variables equals 2 and pols is the list
    of polynomials, then the call **store_quaddobl_system(pols, nbvar=2)**
    will store the polynomials in pols in the quaddobl systems container.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_quaddobl_system
    from phcpy.phcpy2c3 \
        import py2c_syscon_initialize_number_of_quaddobl_polynomials
    from phcpy.phcpy2c3 import py2c_syscon_store_quaddobl_polynomial
    py2c_syscon_clear_quaddobl_system()
    dim = len(polsys)
    fail = 0
    py2c_syscon_initialize_number_of_quaddobl_polynomials(dim)
    for cnt in range(0, dim):
        pol = polsys[cnt]
        nchar = len(pol)
        if(len(nbvar) == 0):
            fail = py2c_syscon_store_quaddobl_polynomial(nchar, dim, cnt+1, pol)
        else:
            nvr = list(nbvar.values())[0]
            fail = py2c_syscon_store_quaddobl_polynomial(nchar, nvr, cnt+1, pol)
        if(fail != 0):
            break
    return fail

def store_multprec_system(polsys, decimals, **nbvar):
    r"""
    Stores the polynomials represented by the list of strings in *polsys*
    into the systems container for multiprecision arithmetic.
    The parameter *decimals* equals the number of decimal places
    in the working precision for the parsing of the strings in *polsys*.
    The number of variables is an optional argument given in *nbvar*.
    If *nbvar* is omitted, then the system is assumed to be square.
    Otherwise, suppose the number of variables equals 2 and pols is the list of
    polynomials, then the call **store_multprec_system(pols, nbvar=2)**
    will store the polynomials in pols in the multiprecision systems container.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_multprec_system
    from phcpy.phcpy2c3 \
        import py2c_syscon_initialize_number_of_multprec_polynomials
    from phcpy.phcpy2c3 import py2c_syscon_store_multprec_polynomial
    py2c_syscon_clear_multprec_system()
    dim = len(polsys)
    fail = 0
    py2c_syscon_initialize_number_of_multprec_polynomials(dim)
    for cnt in range(0, dim):
        pol = polsys[cnt]
        nchar = len(pol)
        if(len(nbvar) == 0):
            fail = py2c_syscon_store_multprec_polynomial\
                       (nchar, dim, cnt+1, decimals, pol)
        else:
            nvr = list(nbvar.values())[0]
            fail = py2c_syscon_store_multprec_polynomial\
                       (nchar, nvr, cnt+1, decimals, pol)
        if(fail != 0):
            break
    return fail

def load_standard_system():
    """
    Returns the polynomials stored in the system container
    for standard double precision arithmetic.
    """
    from phcpy.phcpy2c3 import py2c_syscon_number_of_standard_polynomials
    from phcpy.phcpy2c3 import py2c_syscon_load_standard_polynomial
    dim = py2c_syscon_number_of_standard_polynomials()
    result = []
    for ind in range(1, dim+1):
        result.append(py2c_syscon_load_standard_polynomial(ind))
    return result

def load_dobldobl_system():
    """
    Returns the polynomials stored in the system container
    with double double complex coefficients.
    """
    from phcpy.phcpy2c3 import py2c_syscon_number_of_dobldobl_polynomials
    from phcpy.phcpy2c3 import py2c_syscon_load_dobldobl_polynomial
    dim = py2c_syscon_number_of_dobldobl_polynomials()
    result = []
    for ind in range(1, dim+1):
        result.append(py2c_syscon_load_dobldobl_polynomial(ind))
    return result

def load_quaddobl_system():
    """
    Returns the polynomials stored in the system container
    with quad double complex coefficients.
    """
    from phcpy.phcpy2c3 import py2c_syscon_number_of_quaddobl_polynomials
    from phcpy.phcpy2c3 import py2c_syscon_load_quaddobl_polynomial
    dim = py2c_syscon_number_of_quaddobl_polynomials()
    result = []
    for ind in range(1, dim+1):
        result.append(py2c_syscon_load_quaddobl_polynomial(ind))
    return result

def load_multprec_system():
    """
    Returns the polynomials stored in the system container
    with arbitrary multiprecision complex coefficients.
    """
    from phcpy.phcpy2c3 import py2c_syscon_number_of_multprec_polynomials
    from phcpy.phcpy2c3 import py2c_syscon_load_multprec_polynomial
    dim = py2c_syscon_number_of_multprec_polynomials()
    result = []
    for ind in range(1, dim+1):
        result.append(py2c_syscon_load_multprec_polynomial(ind))
    return result

def store_standard_laurent_system(polsys, **nbvar):
    r"""
    Stores the Laurent polynomials represented by the list of
    strings in *polsys* into the container for systems
    with coefficients in standard double precision.
    If *nbvar* is omitted, then the system is assumed to be square.
    Otherwise, suppose the number of variables equals 2 and pols is the list
    of polynomials, then **store_standard_laurent_system(pols, nbvar=2)**
    stores the polynomials in pols in the standard Laurent systems container.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_standard_Laurent_system
    from phcpy.phcpy2c3 \
        import py2c_syscon_initialize_number_of_standard_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_store_standard_Laurential
    py2c_syscon_clear_standard_Laurent_system()
    dim = len(polsys)
    fail = 0
    py2c_syscon_initialize_number_of_standard_Laurentials(dim)
    for cnt in range(0, dim):
        pol = polsys[cnt]
        nchar = len(pol)
        if len(nbvar) == 0:
            fail = py2c_syscon_store_standard_Laurential(nchar, dim, cnt+1, pol)
        else:
            nvr = list(nbvar.values())[0]
            fail = py2c_syscon_store_standard_Laurential(nchar, nvr, cnt+1, pol)
        if(fail != 0):
            break
    return fail

def store_dobldobl_laurent_system(polsys, **nbvar):
    r"""
    Stores the Laurent polynomials represented by the list of
    strings in *polsys* into the container for systems
    with coefficients in double double precision.
    If *nbvar* is omitted, then the system is assumed to be square.
    Otherwise, suppose the number of variables equals 2 and pols is the list
    of polynomials, then **store_dobldobl_laurent_system(pols, nbvar=2)**
    stores the polynomials in pols in the dobldobl Laurent systems container.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_dobldobl_Laurent_system
    from phcpy.phcpy2c3 \
        import py2c_syscon_initialize_number_of_dobldobl_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_store_dobldobl_Laurential
    py2c_syscon_clear_dobldobl_Laurent_system()
    dim = len(polsys)
    fail = 0
    py2c_syscon_initialize_number_of_dobldobl_Laurentials(dim)
    for cnt in range(0, dim):
        pol = polsys[cnt]
        nchar = len(pol)
        if len(nbvar) == 0:
            fail = py2c_syscon_store_dobldobl_Laurential(nchar, dim, cnt+1, pol)
        else:
            nvr = list(nbvar.values())[0]
            fail = py2c_syscon_store_dobldobl_Laurential(nchar, nvr, cnt+1, pol)
        if(fail != 0):
            break
    return fail

def store_quaddobl_laurent_system(polsys, **nbvar):
    r"""
    Stores the Laurent polynomials represented by the list
    of strings in *polsys* into the container for systems
    with coefficients in quad double precision.
    If *nbvar* is omitted, then the system is assumed to be square.
    Otherwise, suppose the number of variables equals 2 and pols is the list
    of polynomials, then **store_quaddobl_laurent_system(pols, nbvar=2)**
    stores the polynomials in pols in the quaddobl Laurent systems container.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_quaddobl_Laurent_system
    from phcpy.phcpy2c3 \
        import py2c_syscon_initialize_number_of_quaddobl_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_store_quaddobl_Laurential
    py2c_syscon_clear_quaddobl_Laurent_system()
    dim = len(polsys)
    fail = 0
    py2c_syscon_initialize_number_of_quaddobl_Laurentials(dim)
    for cnt in range(0, dim):
        pol = polsys[cnt]
        nchar = len(pol)
        if len(nbvar) == 0:
            fail = py2c_syscon_store_quaddobl_Laurential(nchar, dim, cnt+1, pol)
        else:
            nvr = list(nbvar.values())[0]
            fail = py2c_syscon_store_quaddobl_Laurential(nchar, nvr, cnt+1, pol)
        if(fail != 0):
            break
    return fail

def store_multprec_laurent_system(polsys, decimals, **nbvar):
    r"""
    Stores the Laurent polynomials represented by the list
    of strings in *polsys* into the container for systems
    with coefficients in multiprecision.
    The parameter *decimals* equals the number of decimal places
    in the working precision for the parsing of the strings in *polsys*.
    If *nbvar* is omitted, then the system is assumed to be square.
    Otherwise, suppose the number of variables equals 2 and pols is the list
    of polynomials, then **store_multprec_laurent_system(pols, nbvar=2)**
    stores the polynomials in pols in the multprec Laurent systems container.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_multprec_Laurent_system
    from phcpy.phcpy2c3 \
        import py2c_syscon_initialize_number_of_multprec_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_store_multprec_Laurential
    py2c_syscon_clear_multprec_Laurent_system()
    dim = len(polsys)
    fail = 0
    py2c_syscon_initialize_number_of_multprec_Laurentials(dim)
    for cnt in range(0, dim):
        pol = polsys[cnt]
        nchar = len(pol)
        if len(nbvar) == 0:
            fail = py2c_syscon_store_multprec_Laurential\
                       (nchar, dim, cnt+1, decimals, pol)
        else:
            nvr = list(nbvar.values())[0]
            fail = py2c_syscon_store_multprec_Laurential\
                       (nchar, nvr, cnt+1, decimals, pol)
        if(fail != 0):
            break
    return fail

def load_standard_laurent_system():
    """
    Returns the Laurent polynomials stored in the system container
    for standard double precision arithmetic.
    """
    from phcpy.phcpy2c3 import py2c_syscon_number_of_standard_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_load_standard_Laurential
    dim = py2c_syscon_number_of_standard_Laurentials()
    result = []
    for ind in range(1, dim+1):
        result.append(py2c_syscon_load_standard_Laurential(ind))
    return result

def load_dobldobl_laurent_system():
    """
    Returns the Laurent polynomials stored in the system container
    with double double complex coefficients.
    """
    from phcpy.phcpy2c3 import py2c_syscon_number_of_dobldobl_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_load_dobldobl_Laurential
    dim = py2c_syscon_number_of_dobldobl_Laurentials()
    result = []
    for ind in range(1, dim+1):
        result.append(py2c_syscon_load_dobldobl_Laurential(ind))
    return result

def load_quaddobl_laurent_system():
    """
    Returns the Laurent polynomials stored in the system container
    with quad double complex coefficients.
    """
    from phcpy.phcpy2c3 import py2c_syscon_number_of_quaddobl_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_load_quaddobl_Laurential
    dim = py2c_syscon_number_of_quaddobl_Laurentials()
    result = []
    for ind in range(1, dim+1):
        result.append(py2c_syscon_load_quaddobl_Laurential(ind))
    return result

def load_multprec_laurent_system():
    """
    Returns the Laurent polynomials stored in the system container
    with multiprecision complex coefficients.
    """
    from phcpy.phcpy2c3 import py2c_syscon_number_of_multprec_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_load_multprec_Laurential
    dim = py2c_syscon_number_of_multprec_Laurentials()
    result = []
    for ind in range(1, dim+1):
        result.append(py2c_syscon_load_multprec_Laurential(ind))
    return result

def read_standard_system(filename):
    r"""
    Opens the *filename* for reading a polynomial system
    with coefficients in standard double precision.
    Returns the list of polynomials in the system
    or *None* if something went wrong.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_read_standard_target_system_from_file
    from phcpy.phcpy2c3 import py2c_copy_standard_target_system_to_container
    py2c_syscon_clear_symbol_table()
    lnf = len(filename)
    fail = py2c_read_standard_target_system_from_file(lnf,filename)
    if(fail != 0):
        return None
    else:
        py2c_copy_standard_target_system_to_container()
        return load_standard_system()

def read_dobldobl_system(filename):
    r"""
    Opens the *filename* for reading a polynomial system
    with coefficients in double double precision.
    Returns the list of polynomials in the system
    or *None* if something went wrong.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_read_dobldobl_target_system_from_file
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_target_system_to_container
    py2c_syscon_clear_symbol_table()
    lnf = len(filename)
    fail = py2c_read_dobldobl_target_system_from_file(lnf,filename)
    if(fail != 0):
        return []
    else:
        py2c_copy_dobldobl_target_system_to_container()
        return load_dobldobl_system()

def read_quaddobl_system(filename):
    r"""
    Opens the *filename* for reading a polynomial system
    with coefficients in quad double precision.
    Returns the list of polynomials in the system
    or *None* if something went wrong.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_read_quaddobl_target_system_from_file
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_target_system_to_container
    py2c_syscon_clear_symbol_table()
    lnf = len(filename)
    fail = py2c_read_quaddobl_target_system_from_file(lnf,filename)
    if(fail != 0):
        return []
    else:
        py2c_copy_quaddobl_target_system_to_container()
        return load_quaddobl_system()

def read_standard_system_and_solutions(filename):
    r"""
    Opens the *filename* for reading a polynomial system
    with coefficients in standard double precision,
    and its corresponding list of solutions.
    Returns *None* if the reading went wrong, or otherwise
    returns a tuple with first the list of polynomials
    and second the list of solutions.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_read_standard_start_system_from_file
    from phcpy.phcpy2c3 import py2c_copy_start_system_to_container
    from phcpy.phcpy2c3 import py2c_copy_start_solutions_to_container
    py2c_syscon_clear_symbol_table()
    lnf = len(filename)
    fail = py2c_read_standard_start_system_from_file(lnf,filename)
    if(fail != 0):
        return None
    else:
        py2c_copy_start_system_to_container()
        py2c_copy_start_solutions_to_container()
        return (load_standard_system(), load_standard_solutions())

def read_dobldobl_system_and_solutions(filename):
    r"""
    Opens the *filename* for reading a polynomial system
    with coefficients in double double precision,
    and its corresponding list of solutions.
    Returns *None* if the reading went wrong, or otherwise
    returns a tuple with first the list of polynomials
    and second the list of solutions.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_read_dobldobl_start_system_from_file
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_start_system_to_container
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_start_solutions_to_container
    py2c_syscon_clear_symbol_table()
    lnf = len(filename)
    fail = py2c_read_dobldobl_start_system_from_file(lnf,filename)
    if(fail != 0):
        return None
    else:
        py2c_copy_dobldobl_start_system_to_container()
        py2c_copy_dobldobl_start_solutions_to_container()
        return (load_dobldobl_system(), load_dobldobl_solutions())

def read_quaddobl_system_and_solutions(filename):
    r"""
    Opens the *filename* for reading a polynomial system
    with coefficients in quad double precision,
    and its corresponding list of solutions.
    Returns *None* if the reading went wrong, or otherwise
    returns a tuple with first the list of polynomials
    and second the list of solutions.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_symbol_table
    from phcpy.phcpy2c3 import py2c_read_quaddobl_start_system_from_file
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_start_system_to_container
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_start_solutions_to_container
    py2c_syscon_clear_symbol_table()
    lnf = len(filename)
    fail = py2c_read_quaddobl_start_system_from_file(lnf,filename)
    if(fail != 0):
        return None
    else:
        py2c_copy_quaddobl_start_system_to_container()
        py2c_copy_quaddobl_start_solutions_to_container()
        return (load_quaddobl_system(), load_quaddobl_solutions())

def store_standard_solutions(nvar, sols):
    r"""
    Stores the solutions in the list *sols*, represented as strings
    in PHCpack format into the container for solutions
    with standard double precision.
    The number *nvar* equals the number of variables.
    """
    from phcpy.phcpy2c3 import py2c_solcon_clear_standard_solutions
    from phcpy.phcpy2c3 import py2c_solcon_append_standard_solution_string
    py2c_solcon_clear_standard_solutions()
    fail = 0
    for ind in range(0, len(sols)):
        fail = py2c_solcon_append_standard_solution_string\
                   (nvar, len(sols[ind]), sols[ind])
        if(fail != 0):
            # break
            print('Solution at position', ind, 'is not appended.')
    return fail

def store_dobldobl_solutions(nvar, sols):
    r"""
    Stores the solutions in the list *sols*, represented as strings
    in PHCpack format into the solution container for processing
    with complex double double arithmetic.
    The number *nvar* equals the number of variables.
    """
    from phcpy.phcpy2c3 import py2c_solcon_clear_dobldobl_solutions
    from phcpy.phcpy2c3 import py2c_solcon_append_dobldobl_solution_string
    py2c_solcon_clear_dobldobl_solutions()
    fail = 0
    for ind in range(0, len(sols)):
        fail = py2c_solcon_append_dobldobl_solution_string\
                   (nvar, len(sols[ind]), sols[ind])
        if(fail != 0):
            # break
            print('Solution at position', ind, 'is not appended.')
    return fail

def store_quaddobl_solutions(nvar, sols):
    r"""
    Stores the solutions in the list *sols*, represented as strings
    in PHCpack format into the solution container for processing
    with complex quad double arithmetic.
    The number *nvar* equals the number of variables.
    """
    from phcpy.phcpy2c3 import py2c_solcon_clear_quaddobl_solutions
    from phcpy.phcpy2c3 import py2c_solcon_append_quaddobl_solution_string
    py2c_solcon_clear_quaddobl_solutions()
    fail = 0
    for ind in range(0, len(sols)):
        fail = py2c_solcon_append_quaddobl_solution_string\
                   (nvar, len(sols[ind]), sols[ind])
        if(fail != 0):
            # break
            print('Solution at position', ind, 'is not appended.')
    return fail

def store_multprec_solutions(nvar, sols):
    r"""
    Stores the solutions in the list *sols*, represented as strings
    in PHCpack format into the solution container for processing
    with complex multiprecision arithmetic.
    The number *nvar* equals the number of variables.
    """
    from phcpy.phcpy2c3 import py2c_solcon_clear_multprec_solutions
    from phcpy.phcpy2c3 import py2c_solcon_append_multprec_solution_string
    py2c_solcon_clear_multprec_solutions()
    fail = 0
    for ind in range(0, len(sols)):
        fail = py2c_solcon_append_multprec_solution_string\
                   (nvar, len(sols[ind]), sols[ind])
        if(fail != 0):
            # break
            print('Solution at position', ind, 'is not appended.')
    return fail

def load_standard_solutions():
    """
    Returns the list of solutions stored in the container
    for solutions with standard double precision.
    """
    from phcpy.phcpy2c3 import py2c_solcon_retrieve_next_standard_initialize
    from phcpy.phcpy2c3 import py2c_solcon_move_current_standard_to_next
    from phcpy.phcpy2c3 \
        import py2c_solcon_length_current_standard_solution_string
    from phcpy.phcpy2c3 \
        import py2c_solcon_write_current_standard_solution_string
    result = []
    py2c_solcon_retrieve_next_standard_initialize()
    while True:
        lns = py2c_solcon_length_current_standard_solution_string()
        if(lns == 0):
            break
        sol = py2c_solcon_write_current_standard_solution_string(lns)
        result.append(sol)
        ind = py2c_solcon_move_current_standard_to_next()
        if(ind == 0):
            break
    return result

def load_dobldobl_solutions():
    """
    Returns the list of solutions stored in the container
    for complex double double solutions.
    """
    from phcpy.phcpy2c3 import py2c_solcon_retrieve_next_dobldobl_initialize
    from phcpy.phcpy2c3 import py2c_solcon_move_current_dobldobl_to_next
    from phcpy.phcpy2c3 \
        import py2c_solcon_length_current_dobldobl_solution_string
    from phcpy.phcpy2c3 \
        import py2c_solcon_write_current_dobldobl_solution_string
    result = []
    py2c_solcon_retrieve_next_dobldobl_initialize()
    while True:
        lns = py2c_solcon_length_current_dobldobl_solution_string()
        if(lns == 0):
            break
        sol = py2c_solcon_write_current_dobldobl_solution_string(lns)
        result.append(sol)
        ind = py2c_solcon_move_current_dobldobl_to_next()
        if(ind == 0):
            break
    return result

def load_quaddobl_solutions():
    """
    Returns the list of solutions stored in the container
    for complex quad double solutions.
    """
    from phcpy.phcpy2c3 import py2c_solcon_retrieve_next_quaddobl_initialize
    from phcpy.phcpy2c3 import py2c_solcon_move_current_quaddobl_to_next
    from phcpy.phcpy2c3 \
        import py2c_solcon_length_current_quaddobl_solution_string
    from phcpy.phcpy2c3 \
        import py2c_solcon_write_current_quaddobl_solution_string
    result = []
    py2c_solcon_retrieve_next_quaddobl_initialize()
    while True:
        lns = py2c_solcon_length_current_quaddobl_solution_string()
        if(lns == 0):
            break
        sol = py2c_solcon_write_current_quaddobl_solution_string(lns)
        result.append(sol)
        ind = py2c_solcon_move_current_quaddobl_to_next()
        if(ind == 0):
            break
    return result

def load_multprec_solutions():
    """
    Returns the list of solutions stored in the container
    for complex multiprecision solutions.
    """
    from phcpy.phcpy2c3 import py2c_solcon_retrieve_next_multprec_initialize
    from phcpy.phcpy2c3 import py2c_solcon_move_current_multprec_to_next
    from phcpy.phcpy2c3 \
        import py2c_solcon_length_current_multprec_solution_string
    from phcpy.phcpy2c3 \
        import py2c_solcon_write_current_multprec_solution_string
    result = []
    py2c_solcon_retrieve_next_multprec_initialize()
    while True:
        lns = py2c_solcon_length_current_multprec_solution_string()
        if(lns == 0):
            break
        sol = py2c_solcon_write_current_multprec_solution_string(lns)
        result.append(sol)
        ind = py2c_solcon_move_current_multprec_to_next()
        if(ind == 0):
            break
    return result

def store_standard_witness_set(nbvar, dim, pols, sols):
    r"""
    Given in *nbar* is the total number of variables in the list of
    polynomials in *pols* and its list of solutions in *sols*.
    The coefficients in the polynomials and the coordinates of the
    solutions will be parsed and stored in standard double precision.
    The parameter *dim* equals the number of slack variables used in
    the embedding of *pols* and *sols*.  This *dim* also equals the
    dimension of the solution set represented by the witness set
    given by the lists *pols* and *sols*.
    The symbols for the slack variables are swapped to the end of the
    symbol table in both the polynomials and the solutions.
    """
    from phcpy.phcpy2c3 import py2c_swap_symbols_for_standard_witness_set
    store_standard_system(pols)
    store_standard_solutions(len(pols), sols)
    py2c_swap_symbols_for_standard_witness_set(nbvar, dim)

def store_dobldobl_witness_set(nbvar, dim, pols, sols):
    r"""
    Given in *nbar* is the total number of variables in the list of
    polynomials in *pols* and its list of solutions in *sols*.
    The coefficients in the polynomials and the coordinates of the
    solutions will be parsed and stored in double double precision.
    The parameter *dim* equals the number of slack variables used in
    the embedding of *pols* and *sols*.  This *dim* also equals the
    dimension of the solution set represented by the witness set
    given by the lists *pols* and *sols*.
    The symbols for the slack variables are swapped to the end of the
    symbol table in both the polynomials and the solutions.
    """
    from phcpy.phcpy2c3 import py2c_swap_symbols_for_dobldobl_witness_set
    store_dobldobl_system(pols)
    store_dobldobl_solutions(len(pols), sols)
    py2c_swap_symbols_for_dobldobl_witness_set(nbvar, dim)

def store_quaddobl_witness_set(nbvar, dim, pols, sols):
    r"""
    Given in *nbar* is the total number of variables in the list of
    polynomials in *pols* and its list of solutions in *sols*.
    The coefficients in the polynomials and the coordinates of the
    solutions will be parsed and stored in quad double precision.
    The parameter *dim* equals the number of slack variables used in
    the embedding of *pols* and *sols*.  This *dim* also equals the
    dimension of the solution set represented by the witness set
    given by the lists *pols* and *sols*.
    The symbols for the slack variables are swapped to the end of the
    symbol table in both the polynomials and the solutions.
    """
    from phcpy.phcpy2c3 import py2c_swap_symbols_for_quaddobl_witness_set
    store_quaddobl_system(pols)
    store_quaddobl_solutions(len(pols), sols)
    py2c_swap_symbols_for_quaddobl_witness_set(nbvar, dim)

def store_standard_laurent_witness_set(nbvar, dim, pols, sols):
    r"""
    Given in *nbar* is the total number of variables in the list of
    Laurent polynomials in *pols* and its list of solutions in *sols*.
    The coefficients in the Laurent polynomials and the coordinates of the
    solutions will be parsed and stored in standard double precision.
    The parameter *dim* equals the number of slack variables used in
    the embedding of *pols* and *sols*.  This *dim* also equals the
    dimension of the solution set represented by the witness set
    given by the lists *pols* and *sols*.
    The symbols for the slack variables are swapped to the end of the
    symbol table in both the Laurent polynomials and the solutions.
    """
    from phcpy.phcpy2c3 \
    import py2c_swap_symbols_for_standard_Laurent_witness_set
    store_standard_laurent_system(pols)
    store_standard_solutions(len(pols), sols)
    py2c_swap_symbols_for_standard_Laurent_witness_set(nbvar, dim)

def store_dobldobl_laurent_witness_set(nbvar, dim, pols, sols):
    r"""
    Given in *nbar* is the total number of variables in the list of
    Laurent polynomials in *pols* and its list of solutions in *sols*.
    The coefficients in the Laurent polynomials and the coordinates of the
    solutions will be parsed and stored in double double precision.
    The parameter *dim* equals the number of slack variables used in
    the embedding of *pols* and *sols*.  This *dim* also equals the
    dimension of the solution set represented by the witness set
    given by the lists *pols* and *sols*.
    The symbols for the slack variables are swapped to the end of the
    symbol table in both the Laurent polynomials and the solutions.
    """
    from phcpy.phcpy2c3 \
    import py2c_swap_symbols_for_dobldobl_Laurent_witness_set
    store_dobldobl_laurent_system(pols)
    store_dobldobl_solutions(len(pols), sols)
    py2c_swap_symbols_for_dobldobl_Laurent_witness_set(nbvar, dim)

def store_quaddobl_laurent_witness_set(nbvar, dim, pols, sols):
    r"""
    Given in *nbar* is the total number of variables in the list of
    Laurent polynomials in *pols* and its list of solutions in *sols*.
    The coefficients in the Laurent polynomials and the coordinates of the
    solutions will be parsed and stored in quad double precision.
    The parameter *dim* equals the number of slack variables used in
    the embedding of *pols* and *sols*.  This *dim* also equals the
    dimension of the solution set represented by the witness set
    given by the lists *pols* and *sols*.
    The symbols for the slack variables are swapped to the end of the
    symbol table in both the Laurent polynomials and the solutions.
    """
    from phcpy.phcpy2c3 \
    import py2c_swap_symbols_for_quaddobl_Laurent_witness_set
    store_quaddobl_laurent_system(pols)
    store_quaddobl_solutions(len(pols), sols)
    py2c_swap_symbols_for_quaddobl_Laurent_witness_set(nbvar, dim)

def read_standard_solutions(filename):
    """
    Returns the list of solutions stored on file with the given file name.
    The solutions are parsed in standard double precision.
    """
    from phcpy.phcpy2c3 import py2c_solcon_read_standard_solutions_from_file
    namelen = len(filename)
    py2c_solcon_read_standard_solutions_from_file(namelen, filename)
    return load_standard_solutions()

def read_dobldobl_solutions(filename):
    """
    Returns the list of solutions stored on file with the given file name.
    The solutions are parsed in double double precision.
    """
    from phcpy.phcpy2c3 import py2c_solcon_read_dobldobl_solutions_from_file
    namelen = len(filename)
    py2c_solcon_read_dobldobl_solutions_from_file(namelen, filename)
    return load_dobldobl_solutions()

def read_quaddobl_solutions(filename):
    """
    Returns the list of solutions stored on file with the given file name.
    The solutions are parsed in quad double precision.
    """
    from phcpy.phcpy2c3 import py2c_solcon_read_quaddobl_solutions_from_file
    namelen = len(filename)
    py2c_solcon_read_quaddobl_solutions_from_file(namelen, filename)
    return load_quaddobl_solutions()

def test(prc='d', laurent=False):
    """
    Tests the storing of a witness set for the twisted cubic.
    The embedding induces the order x, y, zz1, z on the variables.
    After storing the witness set, the order is x, y, z, zz1,
    in both the system and solutions.
    The default precision prc is double 'd'.
    Other supported precisions are double double 'dd' and quad double 'qd'.
    """
    print('testing the storing of a witness set for the twisted cubic')
    e0 = 'x^2 - y + (9.22092060529474E-01 + 3.86970582743067E-01*i)*zz1;'
    e1 = 'x^3 - z + (6.47649182027044E-01-7.61938670117025E-01*i)*zz1;'
    e2 = 'zz1;'
    e3 = ' + (-7.98273302152795E-01-6.02295388551227E-01*i)*x' + \
         ' + (-9.99580934345184E-01 + 2.89474643727148E-02*i)*y' + \
         ' + (9.36299591763384E-01 + 3.51202326962281E-01*i)*z' + \
         ' + (8.34685978078231E-01-5.50726173338063E-01*i)*zz1' + \
         ' + (-6.91333588932132E-01 + 7.22535721479719E-01*i);'
    pols = [e0, e1, e2, e3]
    s0 = \
"""
t :  1.00000000000000E+00   0.00000000000000E+00
m : 1
the solution for t :
 x :  1.68397283871286E+00  -3.40775544483616E-01
 y :  2.71963654980455E+00  -1.14771352201599E+00
 zz1 : -2.76305054422513E-32  -1.43165074087146E-32
 z :  4.18868138066541E+00  -2.85950402375559E+00
== err :  1.663E-15 = rco :  1.823E-02 = res :  2.109E-15 =
"""
    s1 = \
"""
t :  1.00000000000000E+00   0.00000000000000E+00
m : 1
the solution for t :
 x : -7.91596946923866E-01  -6.16514124459453E-01
 y :  2.46536060721179E-01   9.76061397315087E-01
 zz1 :  0.00000000000000E+00   0.00000000000000E+00
 z :  4.06598444810859E-01  -9.24640185748066E-01
== err :  2.293E-16 = rco :  8.842E-02 = res :  2.776E-16 =
"""
    s2 = \
"""
t :  1.00000000000000E+00   0.00000000000000E+00
m : 1
the solution for t :
 x :  3.33649121255065E-02   5.79131019739152E-01
 y : -3.34279520662967E-01   3.86453111655036E-02
 zz1 : -2.66882752423716E-33   9.35041489367531E-33
 z : -3.35339052956912E-02  -1.92302242268359E-01
== err :  1.570E-16 = rco :  8.857E-02 = res :  1.457E-16 =
"""
    sols = [s0[1:-1], s1[1:-1], s2[1:-1]]
    if laurent:
        if prc == 'd':
            store_standard_laurent_system(pols)
            pols = load_standard_laurent_system()
        elif prc == 'dd':
            store_dobldobl_laurent_system(pols)
            pols = load_dobldobl_laurent_system()
        elif prc == 'qd':
            store_quaddobl_laurent_system(pols)
            pols = load_quaddobl_laurent_system()
        else:
            print('invalid precision level')
    else:
        if prc == 'd':
            store_standard_system(pols)
            pols = load_standard_system()
        elif prc == 'dd':
            store_dobldobl_system(pols)
            pols = load_dobldobl_system()
        elif prc == 'qd':
            store_quaddobl_system(pols)
            pols = load_quaddobl_system()
        else:
            print('invalid precision level')
    print('the embedded polynomials :')
    for pol in pols:
        print(pol)
    print('the generic points :')
    for sol in sols:
        print(sol)
    input('hit enter to continue')
    if laurent:
        if prc == 'd':
            store_standard_laurent_witness_set(4, 1, pols, sols)
            storedpols = load_standard_laurent_system()
        elif prc == 'dd':
            store_dobldobl_laurent_witness_set(4, 1, pols, sols)
            storedpols = load_dobldobl_laurent_system()
        elif prc == 'qd':
            store_quaddobl_laurent_witness_set(4, 1, pols, sols)
            storedpols = load_quaddobl_laurent_system()
        else:
            print('invalid precision level')
    else:
        if prc == 'd':
            store_standard_witness_set(4, 1, pols, sols)
            storedpols = load_standard_system()
        elif prc == 'dd':
            store_dobldobl_witness_set(4, 1, pols, sols)
            storedpols = load_dobldobl_system()
        elif prc == 'qd':
            store_quaddobl_witness_set(4, 1, pols, sols)
            storedpols = load_quaddobl_system()
        else:
            print('invalid precision level')
    for pol in storedpols:
        print(pol)
    if prc == 'd':
        storedsols = load_standard_solutions()
    elif prc == 'dd':
        storedsols = load_dobldobl_solutions()
    elif prc == 'qd':
        storedsols = load_quaddobl_solutions()
    else:
        print('invalid precision level')
    for sol in storedsols:
        print(sol)

def test_tableau():
    """
    Tests on storing and loading of a tableau.
    """
    from phcpy.phcpy2c3 import py2c_syscon_random_system
    from phcpy.phcpy2c3 import py2c_syscon_clear_standard_system
    dim, nbrmon, deg, cff = 2, 4, 3, 0
    py2c_syscon_random_system(dim, nbrmon, deg, cff)
    pols = load_standard_system()
    print('a random polynomial system :\n', pols)
    neq, nvr, ptab = load_standard_tableau()
    print('its tableau form :\n', ptab)
    py2c_syscon_clear_standard_system()
    store_standard_tableau(ptab)
    storedpols = load_standard_system()
    print('after clearing the container :\n', storedpols)

if __name__ == "__main__":
    test('d')
    test('d', True)
    test('dd')
    test('dd', True)
    test('qd')
    test('qd', True)



"""
Computes all tangent lines to four mutually touching spheres.

The original formulation as polynomial system came from
Cassiano Durand, then at the CS department in Purdue.
The positioning of the centers of the spheres, each with radius
0.5 at the vertices of a tetrahedron came from Thorsten Theobald,
then at TU Muenich.  The centers of the four spheres are

 c1 = (0, 0, 0)
 c2 = (1, 0, 0);
 c3 = (1/2, sqrt(3)/2, 0)
 c4 = (1/2, sqrt(3)/6, sqrt(6)/3);   

Tangent vector t = (x0,x1,x2) and moment vector m = (x3,x4,x5).
The first equation is ||t||=1, the second m.t = 0,
the other equations are ||m - c_i x t ||^2 - r^2 = 0, where
the radius r = 1/2.
"""
from sympy import var, sqrt
from sympy.vector import CoordSys3D, Vector
from phcpy.solver import solve

N = CoordSys3D('N')
x0, x1, x2 = var('x0, x1, x2')
# tangent vector
vt = Vector.zero + x0*N.i + x1*N.j + x2*N.k
normt = vt.dot(vt) - 1
x3, x4, x5 = var('x3, x4, x5')
# moment vector
vm = Vector.zero + x3*N.i + x4*N.j + x5*N.k
momvt = vt.dot(vm)
# centers and radii
ctr1 = (0, 0, 0)
ctr2 = (1, 0, 0)
ctr3 = (0.5, sqrt(3.0)/2, 0)
ctr4 = (0.5, sqrt(3.0)/6, sqrt(6.0)/3)
radius = 0.5
centers = [ctr1, ctr2, ctr3, ctr4]
radii = [radius for _ in range(4)]
eqs = [normt, momvt]
for (ctr, rad) in zip(centers, radii):
    vc = Vector.zero + ctr[0]*N.i + ctr[1]*N.j + ctr[2]*N.k
    left = vm - vc.cross(vt)
    equ = left.dot(left) - rad**2
    eqs.append(equ)
fourspheres = []
print('the polynomial system :')
for pol in eqs:
    print(pol.expand())
    fourspheres.append(str(pol.expand()) + ';')
print('calling the blackbox solver ..')
sols = solve(fourspheres)
print('the solutions :')
for (idx, sol) in enumerate(sols):
    print('Solution', idx+1, ':')
    print(sol)



"""
The module tropisms exports functions to manage numerically computed
tropisms in double, double double, or quad double precision,
in a polyhedral end game with aposteriori step size control.
"""
from ctypes import c_int32, c_double, pointer
from random import randint, uniform
from phcpy.version import get_phcfun
from phcpy.trackers import set_parameter_value, show_parameters
from phcpy.trackers import double_track
from phcpy.trackers import double_double_track, quad_double_track
from phcpy.solver import solve

def double_initialize_tropisms(nbt, dim, wnd, dirs, errs, vrblvl=0):
    r"""
    Initializes the tropisms, given in double precision,
    along with estimates for their winding numbers and errors.
    On entry are the following five parameters:

    *nbt*: the number of direction vectors;

    *dim*: the number of coordinates in each vector;

    *wnd*: a list of integer values for the winding numbers, as many as *nbt*;

    *dirs*: a list of lists of doubles with the coordinates of the directions,
    each inner list has *dim* doubles and *nbt* vectors are given;

    *errs*: a list of *nbt* doubles.
    """
    if vrblvl > 0:
        print('in double_initialize_tropisms, nbt :', nbt, end='')
        print(', dim :', dim)
        print('the winding numbers :', wnd)
        print('the coordinates of tropisms :')
        for direction in dirs:
            print(direction)
        print('the errors :', errs)
    phc = get_phcfun(vrblvl-1)
    pars = (c_int32 * 2)()
    pars[0] = nbt
    pars[1] = dim
    apars = pointer(pars)
    bwnd = (c_int32 * nbt)()
    for (idx, nbr) in enumerate(wnd):
        bwnd[idx] = c_int32(nbr)
    pwnd = pointer(bwnd)
    size = nbt*(dim+1)
    cffs = (c_double * size)()
    idx = 0
    for direction in dirs:
        for nbr in direction:
            cffs[idx] = c_double(nbr)
            idx = idx + 1
    for (idx, nbr) in enumerate(errs):
        cffs[nbt*dim+idx] = c_double(nbr)
    pcffs = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_initialize_tropisms calls phc ...')
    retval = phc(711, apars, pwnd, pcffs, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    return retval

def double_double_initialize_tropisms(nbt, dim, wnd, dirs, errs, vrblvl=0):
    r"""
    Initializes the tropisms, given in double double precision,
    along with estimates for their winding numbers and errors.
    On entry are the following five parameters:

    *nbt*: the number of direction vectors;

    *dim*: the number of coordinates in each vector;

    *wnd*: a list of integer values for the winding numbers, as many as *nbt*;

    *dirs*: a list of lists of doubles with the coordinates of the directions,
    each inner list has *dim* doubles and *nbt* vectors are given;

    *errs*: a list of *nbt* doubles.
    """
    if vrblvl > 0:
        print('in double_double_initialize_tropisms, nbt :', nbt, end='')
        print(', dim :', dim)
        print('the winding numbers :', wnd)
        print('the coordinates of tropisms :')
        for direction in dirs:
            print(direction)
        print('the errors :', errs)
    phc = get_phcfun(vrblvl-1)
    pars = (c_int32 * 2)()
    pars[0] = nbt
    pars[1] = dim
    apars = pointer(pars)
    bwnd = (c_int32 * nbt)()
    for (idx, nbr) in enumerate(wnd):
        bwnd[idx] = c_int32(nbr)
    pwnd = pointer(bwnd)
    size = 2*nbt*(dim+1)
    cffs = (c_double * size)()
    idx = 0
    for direction in dirs:
        for nbr in direction:
            cffs[idx] = c_double(nbr)
            idx = idx + 1
    for (idx, nbr) in enumerate(errs):
        cffs[2*nbt*dim+idx] = c_double(nbr)
    pcffs = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_initialize_tropisms calls phc ...')
    retval = phc(712, apars, pwnd, pcffs, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    return retval

def quad_double_initialize_tropisms(nbt, dim, wnd, dirs, errs, vrblvl=0):
    r"""
    Initializes the tropisms, given in quad double precision,
    along with estimates for their winding numbers and errors.
    On entry are the following five parameters:

    *nbt*: the number of direction vectors;

    *dim*: the number of coordinates in each vector;

    *wnd*: a list of integer values for the winding numbers, as many as *nbt*;

    *dirs*: a list of lists of doubles with the coordinates of the directions,
    each inner list has *dim* doubles and *nbt* vectors are given;

    *errs*: a list of *nbt* doubles.
    """
    if vrblvl > 0:
        print('in quad_double_initialize_tropisms, nbt :', nbt, end='')
        print(', dim :', dim)
        print('the winding numbers :', wnd)
        print('the coordinates of tropisms :')
        for direction in dirs:
            print(direction)
        print('the errors :', errs)
    phc = get_phcfun(vrblvl-1)
    pars = (c_int32 * 2)()
    pars[0] = nbt
    pars[1] = dim
    apars = pointer(pars)
    bwnd = (c_int32 * nbt)()
    for (idx, nbr) in enumerate(wnd):
        bwnd[idx] = c_int32(nbr)
    pwnd = pointer(bwnd)
    size = 4*nbt*(dim+1)
    cffs = (c_double * size)()
    idx = 0
    for direction in dirs:
        for nbr in direction:
            cffs[idx] = c_double(nbr)
            idx = idx + 1
    for (idx, nbr) in enumerate(errs):
        cffs[4*nbt*dim+idx] = c_double(nbr)
    pcffs = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_initialize_tropisms calls phc ...')
    retval = phc(713, apars, pwnd, pcffs, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    return retval

def double_tropisms_number(vrblvl=0):
    """
    Returns the number of tropisms in double precision.
    """
    if vrblvl > 0:
        print('in double_tropisms_number')
    phc = get_phcfun(vrblvl-1)
    apar = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_tropisms_number calls phc ...')
    retval = phc(720, apar, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the number of tropisms :', apar[0])
    return apar[0]

def double_double_tropisms_number(vrblvl=0):
    """
    Returns the number of tropisms in double double precision.
    """
    if vrblvl > 0:
        print('in double_double_tropisms_number')
    phc = get_phcfun(vrblvl-1)
    apar = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_tropisms_number calls phc ...')
    retval = phc(721, apar, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the number of tropisms :', apar[0])
    return apar[0]

def quad_double_tropisms_number(vrblvl=0):
    """
    Returns the number of tropisms in quad double precision.
    """
    if vrblvl > 0:
        print('in quad_double_tropisms_number')
    phc = get_phcfun(vrblvl-1)
    apar = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_tropisms_number calls phc ...')
    retval = phc(722, apar, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the number of tropisms :', apar[0])
    return apar[0]

def double_tropisms_dimension(vrblvl=0):
    """
    Returns the dimension of tropisms in double precision.
    """
    if vrblvl > 0:
        print('in double_tropisms_dimension')
    phc = get_phcfun(vrblvl-1)
    apar = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_tropisms_dimension calls phc ...')
    retval = phc(729, apar, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the dimension of tropisms :', apar[0])
    return apar[0]

def double_double_tropisms_dimension(vrblvl=0):
    """
    Returns the dimension of tropisms in double double precision.
    """
    if vrblvl > 0:
        print('in double_double_tropisms_dimension')
    phc = get_phcfun(vrblvl-1)
    apar = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_tropisms_dimension calls phc ...')
    retval = phc(730, apar, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the dimension of tropisms :', apar[0])
    return apar[0]

def quad_double_tropisms_dimension(vrblvl=0):
    """
    Returns the dimension of tropisms in quad double precision.
    """
    if vrblvl > 0:
        print('in quad_double_tropisms_dimension')
    phc = get_phcfun(vrblvl-1)
    apar = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_tropisms_dimension calls phc ...')
    retval = phc(731, apar, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the dimension of tropisms :', apar[0])
    return apar[0]

def get_double_tropisms(nbt, dim, vrblvl=0):
    r"""
    Given on input the number of tropisms in *nbt* and the dimension in *dim*,
    returns a tuple of three lists: the winding numbers, coordinates of
    the direction vectors, and the errors; in double precision.
    """
    if vrblvl > 0:
        print('in get_double_tropisms, nbt :', nbt, end='')
        print(', dim :', dim)
    phc = get_phcfun(vrblvl-1)
    pars = (c_int32 * 2)()
    pars[0] = nbt
    pars[1] = dim
    apars = pointer(pars)
    bwnd = (c_int32 * nbt)()
    for idx in range(nbt):
        bwnd[idx] = c_int32(0)
    pwnd = pointer(bwnd)
    size = nbt*(dim+1)
    cffs = (c_double * size)()
    for idx in range(size):
        cffs[idx] = c_double(0.0)
    pcffs = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_tropisms calls phc ...')
    retval = phc(717, apars, pwnd, pcffs, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    vals = pwnd[0:nbt]
    wndnbrs = []
    for idx in range(nbt):
        wndnbrs.append(int(vals[0][idx]))
    vals = pcffs[0:size]
    directions = []
    idx = 0
    for _ in range(nbt):
        direction = []
        for _ in range(dim):
            direction.append(float(vals[0][idx]))
            idx = idx + 1
        directions.append(direction)
    errors = []
    for idx in range(size-dim+1, size):
        errors.append(float(vals[0][idx]))
    if vrblvl > 0:
        print('winding numbers :', wndnbrs)
        print('directions :')
        for direction in directions:
            print(direction)
        print('errors :', errors)
    return (wndnbrs, directions, errors)

def get_double_double_tropisms(nbt, dim, vrblvl=0):
    r"""
    Given on input the number of tropisms in *nbt* and the dimension in *dim*,
    returns a tuple of three lists: the winding numbers, coordinates of
    the direction vectors, and the errors; in double double precision.
    """
    if vrblvl > 0:
        print('in get_double_double_tropisms, nbt :', nbt, end='')
        print(', dim :', dim)
    phc = get_phcfun(vrblvl-1)
    pars = (c_int32 * 2)()
    pars[0] = nbt
    pars[1] = dim
    apars = pointer(pars)
    bwnd = (c_int32 * nbt)()
    for idx in range(nbt):
        bwnd[idx] = c_int32(0)
    pwnd = pointer(bwnd)
    size = 2*nbt*(dim+1)
    cffs = (c_double * size)()
    for idx in range(size):
        cffs[idx] = c_double(0.0)
    pcffs = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_double_tropisms calls phc ...')
    retval = phc(718, apars, pwnd, pcffs, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    vals = pwnd[0:nbt]
    wndnbrs = []
    for idx in range(nbt):
        wndnbrs.append(int(vals[0][idx]))
    vals = pcffs[0:size]
    directions = []
    idx = 0
    for _ in range(nbt):
        direction = []
        for _ in range(2*dim):
            direction.append(float(vals[0][idx]))
            idx = idx + 1
        directions.append(direction)
    errors = []
    for idx in range(size-2*dim+2, size):
        errors.append(float(vals[0][idx]))
    if vrblvl > 0:
        print('winding numbers :', wndnbrs)
        print('directions :')
        for direction in directions:
            print(direction)
        print('errors :', errors)
    return (wndnbrs, directions, errors)

def get_quad_double_tropisms(nbt, dim, vrblvl=0):
    r"""
    Given on input the number of tropisms in *nbt* and the dimension in *dim*,
    returns a tuple of three lists: the winding numbers, coordinates of
    the direction vectors, and the errors; in quad double precision.
    """
    if vrblvl > 0:
        print('in get_quad_double_tropisms, nbt :', nbt, end='')
        print(', dim :', dim)
    phc = get_phcfun(vrblvl-1)
    pars = (c_int32 * 2)()
    pars[0] = nbt
    pars[1] = dim
    apars = pointer(pars)
    bwnd = (c_int32 * nbt)()
    for idx in range(nbt):
        bwnd[idx] = c_int32(0)
    pwnd = pointer(bwnd)
    size = 4*nbt*(dim+1)
    cffs = (c_double * size)()
    for idx in range(size):
        cffs[idx] = c_double(0.0)
    pcffs = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_quad_double_tropisms calls phc ...')
    retval = phc(719, apars, pwnd, pcffs, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    vals = pwnd[0:nbt]
    wndnbrs = []
    for idx in range(nbt):
        wndnbrs.append(int(vals[0][idx]))
    vals = pcffs[0:size]
    directions = []
    idx = 0
    for _ in range(nbt):
        direction = []
        for _ in range(4*dim):
            direction.append(float(vals[0][idx]))
            idx = idx + 1
        directions.append(direction)
    errors = []
    for idx in range(size-4*dim+4, size):
        errors.append(float(vals[0][idx]))
    if vrblvl > 0:
        print('winding numbers :', wndnbrs)
        print('directions :')
        for direction in directions:
            print(direction)
        print('errors :', errors)
    return (wndnbrs, directions, errors)

def clear_double_tropisms(vrblvl=0):
    """
    Clears the tropisms in double precision.
    """
    if vrblvl > 0:
        print('in clear_double_tropisms')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_tropisms calls phc ...')
    retval = phc(726, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    return retval

def clear_double_double_tropisms(vrblvl=0):
    """
    Clears the tropisms in double double precision.
    """
    if vrblvl > 0:
        print('in clear_double_double_tropisms')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_double_tropisms calls phc ...')
    retval = phc(727, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    return retval

def clear_quad_double_tropisms(vrblvl=0):
    """
    Clears the tropisms in quad double precision.
    """
    if vrblvl > 0:
        print('in clear_quad_double_tropisms')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_quad_double_tropisms calls phc ...')
    retval = phc(728, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    return retval

def test_double_tropisms_data(vrblvl=0):
    """
    Tests tropisms data in double precision.
    """
    if vrblvl > 0:
        print('in test_double_tropisms_data ...')
    nbr = 2
    dim = 3
    wnd = [randint(10, 99) for _ in range(nbr)]
    dirs = []
    for _ in range(nbr):
        dirs.append([uniform(-1, +1) for _ in range(dim)])
    errs = [uniform(0,1)/1000.0 for _ in range(nbr)]
    if vrblvl > 0:
        print('random winding numbers :', wnd)
        for k in range(len(dirs)):
            print('direction', k+1, ':', dirs[k])
        print('random errors :', errs)
    double_initialize_tropisms(nbr, dim, wnd, dirs, errs, vrblvl)
    size = double_tropisms_number(vrblvl)
    if vrblvl > 0:
        print('number of tropisms :', size)
    fail = int(size != nbr)
    tdim = double_tropisms_dimension(vrblvl)
    if vrblvl > 0:
        print('dimension of tropisms :', tdim)
    fail = fail + int(tdim != dim)
    (rwnd, rdirs, rerrs) = get_double_tropisms(nbr, dim, vrblvl)
    if vrblvl > 0:
        print('retrieved winding numbers :', rwnd)
    for (awnd, bwnd) in zip(wnd, rwnd):
        fail = fail + int(awnd != bwnd)
    if vrblvl > 0:
        print('retrieved directions :')
        for (idx, direction) in enumerate(rdirs):
            print('direction', idx+1, ':', direction)
    for (adir, bdir) in zip(dirs, rdirs):
        for (anbr, bnbr) in zip(adir, bdir):
            fail = fail + int(anbr != bnbr)
    if vrblvl > 0:
        print('retrieved errors :', rerrs)
    for (aerr, berr) in zip(errs, rerrs):
        fail = fail + int(aerr != berr)
    clear_double_tropisms(vrblvl)
    return fail

def test_double_double_tropisms_data(vrblvl=0):
    """
    Tests tropisms data in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_tropisms_data ...')
    nbr = 2
    dim = 3
    wnd = [randint(10, 99) for _ in range(nbr)]
    dirs = []
    for _ in range(nbr):
        dirs.append([uniform(-1, +1) for _ in range(2*dim)])
    errs = [uniform(0,1)/1000.0 for _ in range(2*nbr)]
    if vrblvl > 0:
        print('random winding numbers :', wnd)
        for k in range(len(dirs)):
            print('direction', k+1, ':', dirs[k])
        print('random errors :', errs)
    double_double_initialize_tropisms(nbr, dim, wnd, dirs, errs, vrblvl)
    size = double_double_tropisms_number(vrblvl)
    if vrblvl > 0:
        print('number of tropisms :', size)
    fail = int(size != nbr)
    tdim = double_double_tropisms_dimension(vrblvl)
    if vrblvl > 0:
        print('dimension of tropisms :', tdim)
    fail = fail + int(tdim != dim)
    (rwnd, rdirs, rerrs) = get_double_double_tropisms(nbr, dim, vrblvl)
    if vrblvl > 0:
        print('retrieved winding numbers :', rwnd)
    for (awnd, bwnd) in zip(wnd, rwnd):
        fail = fail + int(awnd != bwnd)
    if vrblvl > 0:
        print('retrieved directions :')
        for (idx, direction) in enumerate(rdirs):
            print('direction', idx+1, ':', direction)
    for (adir, bdir) in zip(dirs, rdirs):
        for (anbr, bnbr) in zip(adir, bdir):
            fail = fail + int(anbr != bnbr)
    if vrblvl > 0:
        print('retrieved errors :', rerrs)
    for (aerr, berr) in zip(errs, rerrs):
        fail = fail + int(aerr != berr)
    clear_double_double_tropisms(vrblvl)
    return fail

def test_quad_double_tropisms_data(vrblvl=0):
    """
    Tests tropisms data in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_tropisms_data ...')
    nbr = 2
    dim = 3
    wnd = [randint(10, 99) for _ in range(nbr)]
    dirs = []
    for _ in range(nbr):
        dirs.append([uniform(-1, +1) for _ in range(4*dim)])
    errs = [uniform(0,1)/1000.0 for _ in range(4*nbr)]
    if vrblvl > 0:
        print('random winding numbers :', wnd)
        for k in range(len(dirs)):
            print('direction', k+1, ':', dirs[k])
        print('random errors :', errs)
    quad_double_initialize_tropisms(nbr, dim, wnd, dirs, errs, vrblvl)
    size = quad_double_tropisms_number(vrblvl)
    if vrblvl > 0:
        print('number of tropisms :', size)
    fail = int(size != nbr)
    tdim = quad_double_tropisms_dimension(vrblvl)
    if vrblvl > 0:
        print('dimension of tropisms :', tdim)
    fail = fail + int(tdim != dim)
    (rwnd, rdirs, rerrs) = get_quad_double_tropisms(nbr, dim, vrblvl)
    if vrblvl > 0:
        print('retrieved winding numbers :', rwnd)
    for (awnd, bwnd) in zip(wnd, rwnd):
        fail = fail + int(awnd != bwnd)
    if vrblvl > 0:
        print('retrieved directions :')
        for (idx, direction) in enumerate(rdirs):
            print('direction', idx+1, ':', direction)
    for (adir, bdir) in zip(dirs, rdirs):
        for (anbr, bnbr) in zip(adir, bdir):
            fail = fail + int(anbr != bnbr)
    if vrblvl > 0:
        print('retrieved errors :', rerrs)
    for (aerr, berr) in zip(errs, rerrs):
        fail = fail + int(aerr != berr)
    clear_quad_double_tropisms(vrblvl)
    return fail

def test_double_endgame(vrblvl=0):
    """
    Tests the numerical computation of a tropism,
    in double precision.
    """
    if vrblvl > 0:
        print('in test_double_endgame ...')
    pols = ['x*z-y*z-z^2+x;', 'z^3-x*y-y*z-z^2-x;', 'x;']
    start = [ 'x*z-y*z-z^2+x;', 'z^3-x*y-y*z-z^2-x;', \
              '(-8.16462369561530E-01 - 5.77398648327108E-01*i)*x' \
           +  '+( 6.63704141654579E-01 - 7.47995195406065E-01*i);']
    startsols = solve(start)
    if vrblvl > 0:
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    if vrblvl > 0:
        print('setting the order of the extrapolator ...')
    set_parameter_value(5, 4, vrblvl-1)
    if vrblvl > 0:
        print('settings of the parameter :')
        show_parameters(vrblvl-1)
    gmm = complex(-0.9669413930172692, 0.25499871072188346)
    gamma, sols = double_track(pols, start, startsols, \
        gamma=gmm, pwt=1, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('gamma :', gamma)
        print('the solutions at the end :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    size = double_tropisms_number(vrblvl-1)
    if vrblvl > 0:
        print('the number of tropisms :', size)
    (wnd, dirs, errs) = get_double_tropisms(len(sols), len(pols), vrblvl)
    if vrblvl > 0:
        print('the winding numbers :', wnd)
        print('the directions :')
        for direction in dirs:
            print(direction)
        print('the errors :', errs)
    return int(sum(wnd) != 10)

def test_double_double_endgame(vrblvl=0):
    """
    Tests the numerical computation of a tropism,
    in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_endgame ...')
    pols = ['x*z-y*z-z^2+x;', 'z^3-x*y-y*z-z^2-x;', 'x;']
    start = [ 'x*z-y*z-z^2+x;', 'z^3-x*y-y*z-z^2-x;', \
              '(-8.16462369561530E-01 - 5.77398648327108E-01*i)*x' \
           +  '+( 6.63704141654579E-01 - 7.47995195406065E-01*i);']
    startsols = solve(start)
    if vrblvl > 0:
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    if vrblvl > 0:
        print('setting the order of the extrapolator ...')
    set_parameter_value(5, 4, vrblvl-1)
    if vrblvl > 0:
        print('settings of the parameter :')
        show_parameters(vrblvl-1)
    gmm = complex(-0.9669413930172692, 0.25499871072188346)
    gamma, sols = double_double_track(pols, start, startsols, \
        gamma=gmm, pwt=1, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('gamma :', gamma)
        print('the solutions at the end :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    size = double_double_tropisms_number(vrblvl-1)
    if vrblvl > 0:
        print('the number of tropisms :', size)
    (wnd, dirs, errs) = get_double_double_tropisms(len(sols), len(pols), \
        vrblvl)
    if vrblvl > 0:
        print('the winding numbers :', wnd)
        print('the directions :')
        for direction in dirs:
            print(direction)
        print('the errors :', errs)
    return int(sum(wnd) != 10)

def test_quad_double_endgame(vrblvl=0):
    """
    Tests the numerical computation of a tropism,
    in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_endgame ...')
    pols = ['x*z-y*z-z^2+x;', 'z^3-x*y-y*z-z^2-x;', 'x;']
    start = [ 'x*z-y*z-z^2+x;', 'z^3-x*y-y*z-z^2-x;', \
              '(-8.16462369561530E-01 - 5.77398648327108E-01*i)*x' \
           +  '+( 6.63704141654579E-01 - 7.47995195406065E-01*i);']
    startsols = solve(start)
    if vrblvl > 0:
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    if vrblvl > 0:
        print('setting the order of the extrapolator ...')
    set_parameter_value(5, 4, vrblvl-1)
    if vrblvl > 0:
        print('settings of the parameter :')
        show_parameters(vrblvl-1)
    gmm = complex(0.1401011540077633, -0.990137195870195)
    gamma, sols = quad_double_track(pols, start, startsols, \
        gamma=gmm, pwt=1, vrblvl=vrblvl-1)
    print(gamma)
    if vrblvl > 0:
        print('the solutions at the end :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    size = quad_double_tropisms_number(vrblvl-1)
    if vrblvl > 0:
        print('the number of tropisms :', size)
    (wnd, dirs, errs) = get_quad_double_tropisms(len(sols), len(pols), \
        vrblvl)
    if vrblvl > 0:
        print('the winding numbers :', wnd)
        print('the directions :')
        for direction in dirs:
            print(direction)
        print('the errors :', errs)
    return int(sum(wnd) != 10)

def main():
    """
    Runs some tests.
    """
    lvl = 1
    fail = test_double_tropisms_data(lvl)
    fail = fail + test_double_double_tropisms_data(lvl)
    fail = fail + test_quad_double_tropisms_data(lvl)
    fail = fail + test_double_endgame(lvl)
    fail = fail + test_double_double_endgame(lvl)
    fail = fail + test_quad_double_endgame(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__== '__main__':
    main()
